@@ -35,8 +35,10 @@ module Text.Regex.TDFA.CorePattern(Q(..),P(..),WhichTest(..),Wanted(..)
                                   ,patternToQ,cleanNullView,cannotAccept,mustAccept) where
 
 import Control.Monad (liftM2, forM, replicateM)
-import Control.Monad.RWS {- all -}
+import Control.Monad.RWS (RWS, runRWS, ask, local, listens, tell, get, put)
+
 import Data.Array.IArray(Array,(!),accumArray,listArray)
+import Data.Either (partitionEithers, rights)
 import Data.List(sort)
 import Data.IntMap.EnumMap2(EnumMap)
 import qualified Data.IntMap.EnumMap2 as Map(singleton,null,assocs,keysSet)
@@ -44,6 +46,7 @@ import qualified Data.IntMap.EnumMap2 as Map(singleton,null,assocs,keysSet)
 import Data.IntSet.EnumSet2(EnumSet)
 import qualified Data.IntSet.EnumSet2 as Set(singleton,toList,isSubsetOf)
 import Data.Semigroup as Sem
+
 import Text.Regex.TDFA.Common {- all -}
 import Text.Regex.TDFA.Pattern(Pattern(..),starTrans)
 -- import Debug.Trace
@@ -269,7 +272,7 @@ cannotAccept q = maybe False (0==) $ snd . takes $ q
 -- favoring pushing Apply into the child postTag makes PGroup happier
 
 type PM = RWS (Maybe GroupIndex) [Either Tag GroupInfo] ([OP]->[OP],Tag)
-type HHQ = HandleTag  -- m1 : info about left boundaary / preTag
+type HHQ = HandleTag  -- m1 : info about left boundary / preTag
         -> HandleTag  -- m2 : info about right boundary / postTag
         -> PM Q
 
@@ -278,18 +281,6 @@ makeGroupArray :: GroupIndex -> [GroupInfo] -> Array GroupIndex [GroupInfo]
 makeGroupArray maxGroupIndex groups = accumArray (\earlier later -> later:earlier) [] (1,maxGroupIndex) filler
     where filler = map (\gi -> (thisIndex gi,gi)) groups
 
-fromRight :: [Either Tag GroupInfo] -> [GroupInfo]
-fromRight [] = []
-fromRight ((Right x):xs) = x:fromRight xs
-fromRight ((Left _):xs) = fromRight xs
-
-partitionEither :: [Either Tag GroupInfo] -> ([Tag],[GroupInfo])
-partitionEither = helper id id where
-  helper :: ([Tag]->[Tag]) -> ([GroupInfo]->[GroupInfo]) -> [Either Tag GroupInfo] -> ([Tag],[GroupInfo])
-  helper ls rs [] = (ls [],rs [])
-  helper ls rs ((Right x):xs) = helper  ls      (rs.(x:)) xs
-  helper ls rs ((Left  x):xs) = helper (ls.(x:)) rs       xs
-
 -- Partial function: assumes starTrans has been run on the Pattern
 -- Note that the lazy dependency chain for this very zigzag:
 --   varies information is sent up the tree
@@ -305,7 +296,7 @@ patternToQ :: CompOption -> (Pattern,(GroupIndex,DoPa)) -> (Q,Array Tag OP,Array
 patternToQ compOpt (pOrig,(maxGroupIndex,_)) = (tnfa,aTags,aGroups) where
   (tnfa,(tag_dlist,nextTag),groups) = runRWS monad startReader startState
   aTags = listArray (0,pred nextTag) (tag_dlist [])
-  aGroups = makeGroupArray maxGroupIndex (fromRight groups)
+  aGroups = makeGroupArray maxGroupIndex (rights groups)
 
   -- implicitly inside a PGroup 0 converted into a GroupInfo 0 undefined 0 1
   monad = go (starTrans pOrig) (Advice 0) (Advice 1)
@@ -354,8 +345,7 @@ patternToQ compOpt (pOrig,(maxGroupIndex,_)) = (tnfa,aTags,aGroups) where
   -- withOrbit uses MonadWriter(listens to makeOrbit/Left), collects
   -- children at all depths
   withOrbit :: PM a -> PM (a,[Tag])
-  withOrbit = listens childStars
-    where childStars x = let (ts,_) = partitionEither x in ts
+  withOrbit = listens $ fst . partitionEithers
 
   {-# INLINE makeGroup #-}
   -- makeGroup usesMonadWriter(tell/Right)
@@ -380,7 +370,7 @@ patternToQ compOpt (pOrig,(maxGroupIndex,_)) = (tnfa,aTags,aGroups) where
   withParent :: GroupIndex -> PM a -> PM (a,[Tag])
   withParent this = local (const (Just this)) . listens childGroupInfo
     where childGroupInfo x =
-            let (_,gs) = partitionEither x
+            let gs = snd $ partitionEithers x
                 children :: [GroupIndex]
                 children = norep . sort . map thisIndex
                            -- filter to get only immediate children (efficiency)
@@ -389,7 +379,7 @@ patternToQ compOpt (pOrig,(maxGroupIndex,_)) = (tnfa,aTags,aGroups) where
 
   -- combineConcat is a partial function: Must not pass in an empty list
   -- Policy choices:
-  --  * pass tags to apply to children and have no preTag or postTag here (so none addded to nullQ)
+  --  * pass tags to apply to children and have no preTag or postTag here (so none added to nullQ)
   --  * middle 'mid' tag: give to left/front child as postTag so a Group there might claim it as a stopTag
   --  * if parent is Group then preReset will become non-empty
   combineConcat :: [Pattern] -> HHQ
@@ -465,7 +455,7 @@ patternToQ compOpt (pOrig,(maxGroupIndex,_)) = (tnfa,aTags,aGroups) where
                newUniq = if needUniqTags then uniq "POr branch" else return bAdvice
 --           trace ("\nPOr sub "++show aAdvice++" "++show bAdvice++"needsTags is "++show needTags) $ return ()
            -- The "bs" values are allocated in left-to-right order before the children in "qs"
-           -- optimiztion: low priority for last branch is implicit, do not create separate tag here.
+           -- optimization: low priority for last branch is implicit, do not create separate tag here.
            bs <- fmap (++[bAdvice]) $ replicateM (pred $ length branches) newUniq -- 2 <= length ps
            -- create all the child branches in left-to-right order after the "bs"
            qs <- forM (zip branches bs) (\(branch,bTag) ->  (go branch aAdvice bTag))
@@ -544,7 +534,7 @@ patternToQ compOpt (pOrig,(maxGroupIndex,_)) = (tnfa,aTags,aGroups) where
          --
          -- If the parent index is Nothing then this is part of a
          -- non-capturing subtree and ignored.  This is a lazy and
-         -- efficient alternative to rebuidling the tree with PGroup
+         -- efficient alternative to rebuilding the tree with PGroup
          -- Nothing replacing PGroup (Just _).
          --
          -- Guarded by the getParentIndex /= Nothing check is the
@@ -593,7 +583,7 @@ Change the NullView to use a tasktags instead of wintags since they are all PreU
          -- PNonEmpty means the child pattern p can be skipped by
          -- bypassing the pattern.  This is only used in the case p
          -- can accept 0 and can accept more than zero characters
-         -- (thus the assertions, enforcted by CorePattern.starTrans).
+         -- (thus the assertions, enforced by CorePattern.starTrans).
          -- The important thing about this case is intercept the
          -- "accept 0" possibility and replace with "skip".
          PNonEmpty p -> mdo
